// Author: Patrick Stickler <patrick.stickler@nokia.com>
// Copyright (C) 2002 Nokia. All rights reserved.
// Usage is governed by the license terms defined in the file
// NOKIA_LICENSE which was distributed with this file.

package Datatypes;

import java.util.HashMap;

/**
 * This is a Java implementation of the datatyping model employed by RDF 1.1.
 *
 * The Java class hierarchy is expected to mirror the RDF subClassOf relations
 * between the datatypes, and the top level concrete datatype classes define
 * a native Java type which is used by all subclasses, to allow for native
 * comparision of hierarchically related values. This is in line with the
 * RDF datatyping model where the class extension of each datatype is its value
 * space and each subordinate datatype must have a value space that is a proper
 * subset of its superordinates -- even though its lexical space and lexical
 * to value mapping need not be compatible.
 *
 */

public abstract class Datatype
{
   protected Object nativeValue = null;
   
   // URI to Datatype Class Registry
   
   private static HashMap registry = new HashMap();
  
   /** Registers datatype Java class name under datatype URI */
   
   public static final void register (String uri, String classname)
   {
      registry.put (uri, classname);
   }
   
   /** Retrieves Java class name associated with specified datatype URI */
   
   public static final String lookup (String uri)
   throws UnknownDatatype
   {
      String className = (String) registry.get (uri);
      if (className == null) throw new UnknownDatatype (uri);
      return className;
   }
   
   /** Creates new datatype value instance of specified datatype,
    *  initialized to the value denoted by the lexical form. */
   
   public static final Datatype value (String uri, String lform)
   throws InvalidLexicalForm, UnknownDatatype, ClassNotFoundException,
   InstantiationException, IllegalAccessException
   {
      // Retrieve datatype class name and create new datatype value instance.
      // Throws UnknownDatatype if no datatype registered under specified URI.
      
      String   classname = lookup (uri);
      Class    datatype  = Class.forName (classname);
            
      Datatype value     = (Datatype) datatype.newInstance ();
      
      // Instantiate value for datatype value object. Throws InvalidLexicalForm
      // if lexical form is not valid for datatype.
      
      value.setValue (lform);
      
      return value;
   }
   
   /** Compares the values denoted by two typed literals, returning 0 if they
    *  are equal, a value less than zero if the first is less than the second, 
    *  and a value greater than zero if the first is greater than the second.
    *
    *  Throws InvalidComparison if datatypes cannot be compared (i.e. one
    *  is not a subClassOf the other) or if either
    *  datatype does not support ordinal comparision of values.
    *  This Datatype implementation requires that subtypes employ the same
    *  underlying native Java type as the root concrete datatype class,
    *  though allowing additional constraints on the value space and
    *  any lexical space and lexical to value mapping whatsoever, so that native
    *  comparisons can be made between values of hierarchically related datatypes. */
   
   public static final int compare (String uriX, String lformX, String uriY, String lformY)
   throws InvalidLexicalForm, UnknownDatatype, InvalidComparison, ClassNotFoundException,
   InstantiationException, IllegalAccessException
   {
      Datatype valueX = value (uriX, lformX);
      Datatype valueY = value (uriY, lformY);
      
      // If datatypes are comparable (one is equal or subordinate to the other),
      // compare values.
      
      return valueX.compareTo (valueY);
   }
   
   /** Compares the values denoted by two typed literals, returning true if they
    *  are equal or false if they are not equal.
    *
    *  Throws NonComparableDatatypes if datatypes cannot be compared (i.e. one
    *  is not a subClassOf the other). */
   
   public static final boolean equal (String uriX, String lformX, String uriY, String lformY)
   throws InvalidLexicalForm, UnknownDatatype, InvalidComparison,
   ClassNotFoundException, InstantiationException, IllegalAccessException
   {
      Datatype valueX = value (uriX, lformX);
      Datatype valueY = value (uriY, lformY);
      
      return valueX.equals (valueY);
   }
   
   /** Compares itself to the specified value, returning 0 if they
    *  are equal, , a value less than zero if less than the specified value, 
    *  and a value greater than zero if greater than the specified value.
    *
    *  Throws InvalidComparison if the datatypes of the values cannot
    *  be compared (i.e. one is not a subclass of the other) or if no
    *  compareTo() method is defined for the datatype class. */
   
   public int compareTo (Datatype value)
   throws InvalidComparison
   {
      throw new InvalidComparison (this.getClass ().getName ()
      + " " + value.getClass ().getName ());
   }
   
   /** Compares itself to the specified value, returning true if they
    *  are equal or false if they are not equal.
    *
    *  Throws InvalidComparison if the datatypes of the values cannot
    *  be compared (i.e. one is not a subclass of the other) or if no
    *  equals() method is defined for the datatype class. */
   
   public boolean equals (Datatype value)
   throws InvalidComparison
   {
      throw new InvalidComparison (this.getClass ().getName ()
      + " " + value.getClass ().getName ());
   }
   
   /** Returns a valid lexical form (string representation) of the value. */
   
   public abstract String lexicalForm ();
   
   /** Sets the value corresponding to the specified lexical form */
   
   public abstract void setValue (String lform)
   throws InvalidLexicalForm;
   
   /** Returns true if lexical form is valid, else returns false. */
   
   public abstract boolean isValidForm (String lform);
}
